@@ -2,6 +2,7 @@ package Jifty::Dispatcher;
 use strict;
 use warnings;
 use Exporter;
+use Jifty::YAML;
 use base qw/Exporter Jifty::Object/;
            
 
@@ -71,8 +72,8 @@ user access to private I<component> templates such as the F<_elements>
 directory in a default Jifty application.  They're also the right way
 to enable L<Jifty::LetMe> actions.
 
-You can entirely stop processing with the C<redirect> and C<abort>
-directives, though L</after> rules will still run.
+You can entirely stop processing with the C<redirect>, C<tangent> and
+C<abort> directives, though L</after> rules will still run.
 
 =item on
 
@@ -82,15 +83,15 @@ place to set up view-specific objects or load up values for your
 templates.
 
 Dispatcher directives are evaluated in order until we get to either a
-C<show>, C<redirect> or an C<abort>.
+C<show>, C<redirect>, C<tangent> or C<abort>.
 
 =item after
 
 L<after> rules let you clean up after rendering your page. Delete your
 cache files, write your transaction logs, whatever.
 
-At this point, it's too late to C<show>, C<redirect> or C<abort> page
-display.
+At this point, it's too late to C<show>, C<redirect>, C<tangent> or C<abort>
+page display.
 
 =back
 
@@ -237,10 +238,16 @@ Break out from the current C<run> block and stop running rules in this stage.
 
 Abort the request; this skips straight to the cleanup stage.
 
+If C<$code> is specified, it's used as the HTTP status code.
+
 =head2 redirect $uri
 
 Redirect to another URI.
 
+=head2 tangent $uri
+
+Take a continuation here, and tangent to another URI.
+
 =head2 plugin
 
 See L</Plugins and rule ordering>, above.
@@ -252,7 +259,7 @@ our @EXPORT = qw<
 
     before on after
 
-    show dispatch abort redirect
+    show dispatch abort redirect tangent
 
     GET POST PUT HEAD DELETE OPTIONS
 
@@ -278,6 +285,7 @@ sub run (&@)      { _ret @_ }    # execute a block of code
 sub show (;$@)    { _ret @_ }    # render a page
 sub dispatch ($@) { _ret @_ }    # run dispatch again with another URI
 sub redirect ($@) { _ret @_ }    # web redirect
+sub tangent ($@)  { _ret @_ }    # web tangent
 sub abort (;$@)   { _ret @_ }    # abort request
 sub default ($$@) { _ret @_ }    # set parameter if it's not yet set
 sub set ($$@)     { _ret @_ }    # set parameter
@@ -673,7 +681,7 @@ sub _do_run {
 
 This method is called by the dispatcher internally. You shouldn't need to.
 
-Redirect the user to the URL provded in the mandatory PATH argument.
+Redirect the user to the URL provided in the mandatory PATH argument.
 
 =cut
 
@@ -683,6 +691,21 @@ sub _do_redirect {
     Jifty->web->redirect($path);
 }
 
+=head2 _do_tangent PATH
+
+This method is called by the dispatcher internally. You shouldn't need to.
+
+Take a tangent to the URL provided in the mandatory PATH argument.
+(See L<Jifty::Manual::Continuation> for more about tangents.)
+
+=cut
+
+sub _do_tangent {
+    my ( $self, $path ) = @_;
+    $self->log->debug("Taking a tangent to $path");
+    Jifty->web->tangent(url => $path);
+}
+
 =head2 _do_abort 
 
 This method is called by the dispatcher internally. You shouldn't need to.
@@ -694,6 +717,16 @@ Don't display any page. just stop.
 sub _do_abort {
     my $self = shift;
     $self->log->debug("Aborting processing");
+    if (@_) {
+        # This is the status code
+        my $status = shift;
+        my $apache = Jifty->handler->apache;
+        $apache->header_out(Status => $status);
+        $apache->send_http_header;
+
+        require HTTP::Status;
+        print STDOUT $status, ' ' , HTTP::Status::status_message($status);
+    }
     $self->_abort;
 }
 
@@ -882,8 +915,8 @@ came in with that method.
 
 sub _match_method {
     my ( $self, $method ) = @_;
-    $self->log->debug("Matching URL ".$self->{cgi}->method." against ".$method);
-    lc( $self->{cgi}->method ) eq lc($method);
+    $self->log->debug("Matching URL $ENV{REQUEST_METHOD} against ".$method);
+    lc( $ENV{REQUEST_METHOD} ) eq lc($method);
 }
 
 sub _match_plugin {
@@ -979,6 +1012,14 @@ All other C<*> can match zero or more non-slash characters:
 
 =item *
 
+Two stars (C<**>) can match zero or more characters, including slash:
+
+    /**/bar
+    /foo/**
+    **
+
+=item *
+
 Consecutive C<?> marks are captured together:
 
     /foo???bar      # One capture for ???
@@ -1005,6 +1046,10 @@ sub _compile_glob {
         \\ \*        # star
         (?= / | \z)  # lookahead for slash or end-of-string
     }{([^/]+)}gx;
+    $glob =~ s{
+        # Two stars can match zero or more characters, including slash.
+        \\ \* \\ \*
+    }{(.*)}gx;
     $glob =~ s{
         # All other stars can match zero or more non-slash character.
         \\ \*
