@@ -85,7 +85,8 @@ sub connect {
     my %db_config =  (%{Jifty->config->framework('Database')}, Database => $self->canonical_database_name);
 
     my %lc_db_config;
-    for (keys %db_config) {
+    # Skip the non-dsn keys, but not anything else
+    for (grep {!/^version|recordbaseclass$/i} keys %db_config) {
         $lc_db_config{lc($_)} = $db_config{$_};
     }
     $self->SUPER::connect( %lc_db_config , %args);
