@@ -186,10 +186,8 @@ sub handle_request {
     $self->stash({});
     local $HTML::Mason::Commands::JiftyWeb = Jifty::Web->new();
 
-    Jifty->web->setup_session;
     Jifty->web->request( Jifty::Request->new()->fill( $self->cgi ) );
     Jifty->web->response( Jifty::Response->new );
-    Jifty->web->session->set_cookie;
     Jifty->api->reset;
     $_->new_request for Jifty->plugins;
 
@@ -199,10 +197,15 @@ sub handle_request {
         = $self->static_handler->handle_request( Jifty->web->request->path )
         if ( Jifty->config->framework('Web')->{'ServeStaticFiles'} );
 
+    Jifty->web->setup_session unless $sent_response;
+
     # Return from the continuation if need be
     Jifty->web->request->return_from_continuation;
 
-    $self->dispatcher->handle_request() unless ($sent_response);
+    unless ($sent_response) {
+        Jifty->web->session->set_cookie;
+        $self->dispatcher->handle_request()
+    }
 
     $self->cleanup_request();
 
