@@ -22,7 +22,7 @@ use base qw/Class::Accessor::Fast Class::Data::Inheritable Jifty::Object/;
 use vars qw/$SERIAL @JS_INCLUDES/;
 
 __PACKAGE__->mk_accessors(
-    qw(next_page request response session temporary_current_user _current_user)
+    qw(next_page force_redirect request response session temporary_current_user _current_user)
 );
 
 __PACKAGE__->mk_classdata($_)
@@ -62,7 +62,7 @@ __PACKAGE__->javascript_libs([qw(
     yui/calendar.js
     app.js
     app_behaviour.js
-    css_browser_selector.js                                 
+    css_browser_selector.js
 )]);
 
 =head1 METHODS
@@ -210,9 +210,6 @@ sub current_user {
         my $currentuser_obj = shift;
         $self->session->set(
             'user_id' => $currentuser_obj ? $currentuser_obj->id : undef );
-        $self->session->set( 'user_ref' => $currentuser_obj
-            ? ref $currentuser_obj->user_object
-            : undef );
         $self->_current_user( $currentuser_obj || undef );
     }
 
@@ -224,11 +221,7 @@ sub current_user {
     } elsif ( my $id = $self->session->get('user_id') ) {
         my $object = (
             Jifty->config->framework('ApplicationClass') . "::CurrentUser" )
-            ->new();
-        my $user
-            = $self->session->get('user_ref')->new( current_user => $object );
-        $user->load_by_cols( id => $id );
-        $object->user_object($user);
+            ->new( id => $id );
         $self->_current_user($object);
         return $object;
     } else {
@@ -387,7 +380,7 @@ sub form {
 
 Creates a new action (an instance of a subclass of L<Jifty::Action>)
 
-C<CLASS> is L<qualified|Jifty::Util/qualify>, and an instance of that
+C<CLASS> is L<qualified|Jifty::API/qualify>, and an instance of that
 class is created, passing the C<Jifty::Web> object, the C<MONIKER>,
 and any other arguments that C<new_action> was supplied.
 
@@ -399,7 +392,7 @@ C<ORDER> defines the order in which the action is run, with lower
 numerical values running first.
 
 C<ARGUMENTS> are passed to the L<Jifty::Action/new> method.  In
-addition, if the current request (C<$self->request>) contains an
+addition, if the current request (C<< $self->request >>) contains an
 action with a matching moniker, any arguments that are in that
 requested action but not in the C<PARAMHASH> list are set.  This
 implements "sticky fields".
@@ -422,7 +415,6 @@ sub new_action {
         @_
     );
 
-    my %arguments = %{ $args{arguments} };
 
     if ( $args{'moniker'} ) {
         my $action_in_request = $self->request->action( $args{moniker} );
@@ -431,7 +423,7 @@ sub new_action {
     # from the request; we read from the request to implement "sticky fields".
     #
         if ( $action_in_request and $action_in_request->arguments ) {
-            %arguments = ( %{ $action_in_request->arguments }, %arguments );
+            $args{'request_arguments'} = $action_in_request->arguments;
         }
     }
 
@@ -453,7 +445,7 @@ sub new_action {
 
     my $action;
     # XXX TODO bullet proof
-    eval { $action = $class->new( %args, arguments => {%arguments} ); };
+    eval { $action = $class->new(%args) };
     if ($@) {
         my $err = $@;
         $self->log->fatal($err);
@@ -528,23 +520,31 @@ sub succeeded_actions {
 Gets or sets the next page for the framework to show.  This is
 normally set during the C<take_action> method or a L<Jifty::Action>
 
+=head3 force_redirect [VALUE]
+
+Gets or sets whether we should force a redirect to C<next_page>, even
+if it's already the current page. You might set this, e.g. to force a
+redirect after a POSTed action.
+
 =head3 redirect_required
 
 Returns true if we need to redirect, now that we've processed all the
-actions.  The current logic just looks to see if a different
-L</next_page> has been set. We probably want to make it possible to
-force a redirect, even if we're redirecting back to the current page
+actions. We need a redirect if either C<next_page> is different from
+the current page, or C<force_redirect> has been set.
 
 =cut
 
 sub redirect_required {
     my $self = shift;
 
-    if ($self->next_page
+    return ( 1 ) if $self->force_redirect;
+
+    if (!$self->request->is_subrequest
+        and $self->next_page
         and ( ( $self->next_page ne $self->request->path )
-            or $self->request->state_variables
-            or $self->{'state_variables'} )
-        )
+              or $self->request->state_variables
+              or $self->{'state_variables'} )
+       )
     {
         return (1);
 
@@ -566,11 +566,11 @@ L<Jifty::Web::Form::Clickable> object.
 
 sub redirect {
     my $self = shift;
-    my $page = shift || $self->next_page;
+    my $page = shift || $self->next_page || $self->request->path;
     $page = Jifty::Web::Form::Clickable->new( url => $page )
       unless ref $page and $page->isa("Jifty::Web::Form::Clickable");
 
-    carp "Don't include GET paramters in the redirect URL -- use a Jifty::Web::Form::Clickable instead.  See L<Jifty::Web/redirect>" if $page->url =~ /\?/;
+    carp "Don't include GET parameters in the redirect URL -- use a Jifty::Web::Form::Clickable instead.  See L<Jifty::Web/redirect>" if $page->url =~ /\?/;
 
     my %overrides = ( @_ );
     $page->parameter($_ => $overrides{$_}) for keys %overrides;
@@ -688,9 +688,6 @@ sub tangent {
     if ( defined wantarray ) {
         return $clickable->generate;
     } else {
-        $clickable->state_variable( $_ => $self->{'state_variables'}{$_} )
-            for keys %{ $self->{'state_variables'} };
-
         my $request = Jifty->web->request->clone;
         my %clickable = $clickable->get_parameters;
         $request->argument($_ => $clickable{$_}) for keys %clickable;
@@ -726,13 +723,16 @@ sub link {
 
 =head3 return PARAMHASH
 
-Generates and renders a L<Jifty::Web::Form::Clickable> using the given
-I<PARAMHASH>, additionally defaults to calling the current
-continuation.
+If called in non-void context, creates and renders a
+L<Jifty::Web::Form::Clickable> using the given I<PARAMHASH>,
+additionally defaults to calling the current continuation.
 
 Takes an additional argument, C<to>, which can specify a default path
 to return to if there is no current continuation.
 
+In void context, does a redirect to the URL that the
+L<Jifty::Web::Form::Clickable> object generates.
+
 =cut
 
 sub return {
@@ -740,11 +740,22 @@ sub return {
     my %args = (@_);
     my $continuation = Jifty->web->request->continuation;
     if (not $continuation and $args{to}) {
-        $continuation = Jifty::Continuation->new(request => Jifty::Request->new(path => $args{to}));
+        $continuation = Jifty::Continuation->new(
+            request => Jifty::Request->new(path => $args{to})
+        );
     }
     delete $args{to};
 
-    $self->link( call => $continuation, %args );
+    my $clickable = Jifty::Web::Form::Clickable->new(
+        call => $continuation, %args
+    );
+
+    if ( defined wantarray ) {
+        return $clickable->generate;
+    }
+    else {
+        $self->redirect($clickable);
+    }
 }
 
 =head3 render_messages [MONIKER]
@@ -823,15 +834,8 @@ sub _render_messages {
     return unless grep {$_->$type()} values %results;
     
     my $plural = $type . "s";
-    $self->out(qq{<div id="$plural">});
+    $self->out(qq{<div class="jifty results messages" id="$plural">});
     
-    $self->out( qq[<a id="dismiss_$plural" href="#" title="]
-               .  _('Dismiss')
-               .qq[" onmousedown="this.onfocus=this.blur;" onmouseup="this.onfocus=window.clientInformation?null:window.undefined" ]
-               .qq[ onclick="Effect.Fade(this.parentNode); return false;">]
-               .  _('Dismiss')
-               .qq[</a>]);
-               
     foreach my $moniker ( keys %results ) {
         if ( $results{$moniker}->$type() ) {
             $self->out( qq{<div class="$type $moniker">}
@@ -867,6 +871,7 @@ HTML-escapes the given string and returns it
 =cut
 
 sub escape {
+    no warnings 'uninitialized';
     my $self = shift;
     return join '', map {my $html = $_; Jifty::View::Mason::Handler::escape_utf8( \$html ); $html} @_;
 }
@@ -878,6 +883,7 @@ URI-escapes the given string and returns it
 =cut
 
 sub escape_uri {
+    no warnings 'uninitialized';
     my $self = shift;
     return join '', map {my $uri = $_; Jifty::View::Mason::Handler::escape_uri( \$uri ); $uri} @_;
 }
@@ -1150,16 +1156,17 @@ sub set_variable {
 =head3 state_variables
 
 Returns all of the state variables that have been set for the next
-request, as a hash; they have already been prefixed with C<J:V->
+request, as a hash;
+
+N.B. These are B<not> prefixed with C<J:V->, as they were in earlier
+versions of Jifty
 
 =cut
 
-# FIXME: it seems wrong to have an accessor that exposes the
-# representation, so to speak
 sub state_variables {
     my $self = shift;
     my %vars;
-    $vars{ "J:V-" . $_ } = $self->{'state_variables'}->{$_}
+    $vars{$_} = $self->{'state_variables'}->{$_}
         for keys %{ $self->{'state_variables'} };
 
     return %vars;
