@@ -17,7 +17,7 @@ the name of the javascript event handler, such as C<onclick>, with a
 set of arguments.
 
 The format of the arguments passed to C<onclick> (or any similar
-method) is a hash reference or string Strings are inserted verbatim.
+method) is a hash reference or string.  Strings are inserted verbatim.
 
 Hash references can take a number of possible keys.  The most
 important is the mode of the fragment replacement, if any; it is
@@ -86,6 +86,15 @@ specifying elements of parent page regions.
 An action, moniker of an action, or array reference to such; these
 actions are submitted when the event is fired.
 
+=item disable => BOOLEAN
+
+If true, disable all form fields associated with the actions in
+C<submit> when this Element is clicked. This serves to give immediate
+visual indication that the request is being processed, as well as to
+prevent double-submits.
+
+Defaults to true.
+
 =item args => HASHREF
 
 Arguments to the region.  These will override the arguments to the
@@ -101,6 +110,14 @@ fragment.
 A hashref of arguments to pass to the effect when it is created.  These
 can be used to change the duration of the effect, for instance.
 
+=item confirm => STRING
+
+Prompt the user with a Javascript confirm dialog with the given text
+before carrying out the rest of the handlers. If the user cancels, do
+nothing, otherwise proceed as normal.
+
+TODO: This does not have a non-Javascript fallback method yet.
+
 =back
 
 =cut
@@ -128,7 +145,76 @@ C<new> parameter hash.
 =cut
 
 sub accessors { shift->handlers, qw(class key_binding id label tooltip) }
-__PACKAGE__->mk_accessors(qw(onclick class key_binding id label tooltip));
+__PACKAGE__->mk_accessors(qw(_onclick class key_binding id label tooltip));
+
+=head2 new PARAMHASH OVERRIDE
+
+Create a new C<Jifty::Web::Form::Element> object blessed with
+PARAMHASH, and set with accessors for the hash values in OVERRIDE.
+
+=cut
+
+sub new {
+    my ($class, $args, $override) = @_;
+    # force using accessor for onclick init
+    if (my $onclick = delete $args->{onclick}) {
+        $override->{onclick} = $onclick;
+    }
+
+    my $self = $class->SUPER::new($args);
+
+    if ($override) {
+        for my $field ( $self->accessors() ) {
+            # XXX: warn about unexpected ones
+            $self->$field( $override->{$field} ) if exists $override->{$field};
+        }
+    }
+
+    return $self;
+}
+
+=head2 onclick
+
+=cut
+
+sub onclick {
+    my $self = shift;
+    return $self->_onclick unless @_;
+
+    my ($arg) = @_;
+
+    $arg = [$arg] unless ref $arg eq 'ARRAY';
+
+    #Normalize arguments as much as possible here, to simplify later
+    #processing of them here and in Clickable.
+    for my $hook ( @$arg ) {
+        next unless ref $hook eq 'HASH';
+
+        # Normalize actions to monikers to prevent circular references,
+        # since Jifty::Action caches instances of Jifty::Web::Form::Clickable.
+        if ( $hook->{submit} ) {
+            $hook->{submit} = [ $hook->{submit} ] unless ref $hook->{submit} eq "ARRAY";
+            $hook->{submit} = [ map { ref $_ ? $_->moniker : $_ } @{ $hook->{submit} } ];
+        }
+
+        if ( $hook->{args} ) {
+            # We're going to pass complex query mapping structures
+            # as-is to the server, but we need to make sure we're not
+            # trying to pass around Actions, merely their monikers.
+            for my $key ( keys %{ $hook->{args} } ) {
+                next unless ref $hook->{args}{$key} eq "HASH";
+                $hook->{args}{$key}{$_} = $hook->{args}{$key}{$_}->moniker
+                  for grep { ref $hook->{args}{$key}{$_} }
+                  keys %{ $hook->{args}{$key} };
+            }
+        } else {
+            $hook->{args} = {};
+        }
+
+    }
+
+    $self->_onclick($arg);
+}
 
 =head2 javascript
 
@@ -145,20 +231,28 @@ sub javascript {
         next unless $value;
 
         my @fragments;
-        my @actions;
+        my %actions;    # Maps actions => disable?
+        my $confirm;
 
-        for my $hook (grep {ref $_ eq "HASH"} (ref $value eq "ARRAY" ? @{$value} : ($value))) {
+        for my $hook (grep {ref $_ eq "HASH"} (@{$value})) {
 
             my %args;
 
             # Submit action
             if ( $hook->{submit} ) {
-                $hook->{submit} = [ $hook->{submit} ] unless ref $hook->{submit} eq "ARRAY";
-                push @actions, map { ref $_ ? $_->moniker : $_ } @{ $hook->{submit} };
+                my $disable = exists $hook->{disable} ? $hook->{disable} : 1;
+                # Normalize to 1/0 to pass to JS
+                $disable = $disable ? 1 : 0;
+                $actions{$_} = $disable for (@{ $hook->{submit} }); 
             }
 
             $hook->{region} ||= Jifty->web->qualified_region;
 
+            # Should we show a javascript confirm message?
+            if ($hook->{confirm}) {
+                $confirm = $hook->{confirm};
+            }
+
             # Placement
             if (exists $hook->{append}) {
                 @args{qw/mode path/} = ('Bottom', $hook->{append});
@@ -199,21 +293,12 @@ sub javascript {
                 $args{region}  = $args{element} =~ /^#region-(\S+)/ ? "$1-".Jifty->web->serial : Jifty->web->serial;
             }
 
+            # Arguments
+            $args{args} = $hook->{args};
+
             # Toggle functionality
             $args{toggle} = 1 if $hook->{toggle};
 
-            # Arguments
-            $args{args} = $hook->{args} || {};
-
-            # We're going to pass complex query mapping structures
-            # as-is to the server, but we need to make sure we're not
-            # trying to pass around Actions, merely their monikers.
-            for my $key (keys %{$args{args}}) {
-                next unless ref $args{args}{$key} eq "HASH";
-                $args{args}{$key}{$_} = $args{args}{$key}{$_}->moniker
-                  for grep {ref $args{args}{$key}{$_}} keys %{$args{args}{$key}};
-            }
-
             # Effects
             $args{$_} = $hook->{$_} for grep {exists $hook->{$_}} qw/effect effect_args/;
 
@@ -221,14 +306,14 @@ sub javascript {
         }
 
         my $string = join ";", (grep {not ref $_} (ref $value eq "ARRAY" ? @{$value} : ($value)));
-        if (@fragments or @actions) {
-            # Uniqify action monikers
-            my %uniq;
-            @actions = grep {not $uniq{$_}++} @actions;
+        if (@fragments or %actions) {
 
-            my $update = "update( ". Jifty::JSON::objToJson( {actions => \@actions, fragments => \@fragments }, {singlequote => 1}) .", this );";
+            my $update = "update( ". Jifty::JSON::objToJson( {actions => \%actions, fragments => \@fragments }, {singlequote => 1}) .", this );";
             $string .= $self->javascript_preempt ? "return $update" : "$update; return true;";
         }
+        if ($confirm) {
+            $string = "if(!confirm(" . Jifty::JSON::objToJson($confirm, {singlequote => 1}) . ")) return false;" . $string;
+        }
         $response .= qq| $trigger="$string"|;
     }
     return $response;
