@@ -8,10 +8,19 @@ Jifty.Calendar = {
         
         if ( !input ) return false;
 
-        DOM.Events.addListener( input, "click", Jifty.Calendar.toggleCalendar );
+        DOM.Events.addListener( input, "focus", Jifty.Calendar.toggleCalendar );
+        DOM.Events.addListener( input, "blur", Jifty.Calendar.doBlur );
         return true;
     },
 
+    dateFormat: "Y-m-d",
+    dateRegex: /^(\d{4})\W(\d{2})\W(\d{2})/,
+    
+    Options: {
+        NAV_ARROW_LEFT: "/static/images/yui/us/tr/callt.gif",
+        NAV_ARROW_RIGHT: "/static/images/yui/us/tr/calrt.gif"
+    },
+
     toggleCalendar: function(ev) {
         var calId  = "cal_" + ev.target.id;
         var wrapId = calId + "_wrap";
@@ -22,21 +31,13 @@ Jifty.Calendar = {
             Jifty.Calendar.hideOpenCalendar();
             return;
         }
-
+        
         Jifty.Calendar.hideOpenCalendar();
         
         /* We need to delay Jifty's canonicalization until after we've
            selected a value via the calendar */
-        input["_onblur"] = input.onblur;
-        input.onblur     = null;
+        Form.Element.disableValidation(input);
         
-        if ( wrap ) {
-            wrap.style.display = "block";
-            Jifty.Calendar.openCalendar = wrapId;
-            Jifty.Utils.scrollToShow( wrapId );
-            return;
-        }
-
         wrap = document.createElement("div");
         wrap.setAttribute( "id", wrapId );
         
@@ -49,8 +50,8 @@ Jifty.Calendar = {
 
         var cal;
         
-        if ( /^(\d{4})-(\d{2})-(\d{2})/.test(input.value) ) {
-            var bits = input.value.match(/^(\d{4})-(\d{2})-(\d{2})/);
+        if (Jifty.Calendar.dateRegex.test(input.value) ) {
+            var bits = input.value.match(Jifty.Calendar.dateRegex);
             cal = new YAHOO.widget.Calendar( calId,
                                              wrapId,
                                              bits[2]+"/"+bits[1],
@@ -61,12 +62,27 @@ Jifty.Calendar = {
             cal = new YAHOO.widget.Calendar( calId, wrapId );
         }
         
+
+        cal["customConfig"] = function(){ 
+            for (i in Jifty.Calendar.Options) {
+                this.Options[i] = Jifty.Calendar.Options[i];
+            } 
+        };
+
         cal["onSelect"] = function() {
-            input.value = cal.getSelectedDates()[0].formatDate("Y-m-d");
+            input.value = cal.getSelectedDates()[0].formatDate(Jifty.Calendar.dateFormat);
             Jifty.Calendar.hideOpenCalendar();
         };
-        cal.render();
+
+        cal["_onChangePage"] = cal["onChangePage"];
+        cal["onChangePage"]  = function() {
+            Jifty.Calendar._blurredCalendar = null;
+            cal["_onChangePage"]();
+        };
         
+        cal.setupConfig();
+        cal.render();
+
         Jifty.Calendar.openCalendar = wrapId;
         Jifty.Utils.scrollToShow( wrapId );
         /*Jifty.Calendar.preventStutter = wrapId;*/
@@ -76,21 +92,39 @@ Jifty.Calendar = {
 
     hideOpenCalendar: function() {
         if ( Jifty.Calendar.openCalendar && $( Jifty.Calendar.openCalendar ) ) {
-            $( Jifty.Calendar.openCalendar ).style.display = "none";
 
             /* Get the input's ID */
             var inputId = Jifty.Calendar.openCalendar;
                 inputId = inputId.replace(/^cal_/, '');
                 inputId = inputId.replace(/_wrap$/, '');
 
+            Element.remove(Jifty.Calendar.openCalendar);
+
             var input = $( inputId );
 
-            /* Restore the original onblur */
-            input.onblur     = input["_onblur"];
-            input["_onblur"] = null;
-            
+            /* Reenable canonicalization */
+            Form.Element.enableValidation(input);
+
             Jifty.Calendar.openCalendar = "";
         }
+    },
+
+    _doneBlurOnce: false,
+    _blurredCalendar: null,
+    doBlur: function(ev) {
+        if ( Jifty.Calendar.openCalendar && !Jifty.Calendar._doneBlurOnce ) {
+            Jifty.Calendar._doneBlurOnce    = true;
+            Jifty.Calendar._blurredCalendar = Jifty.Calendar.openCalendar;
+            setTimeout( Jifty.Calendar.doBlur, 200 );
+            return;
+        }
+        else if ( Jifty.Calendar._doneBlurOnce
+                  && Jifty.Calendar._blurredCalendar == Jifty.Calendar.openCalendar )
+        {
+            Jifty.Calendar.hideOpenCalendar();
+        }
+        Jifty.Calendar._doneBlurOnce    = false;
+        Jifty.Calendar._blurredCalendar = null;
     }
 };
 
